#!/bin/sh

LOG=/tmp/ffmpeg.log
PID=

get_ffmpeg_bin() {
  if [ -f /flash/qvidium/inputs.conf ] ; then
    . /flash/qvidium/inputs.conf
    if [ "$QVDEC_BOARD" = true ] ; then
      FFMPEG_BIN=bin/ffmpeg-2.18
      export LD_LIBRARY_PATH=$FFMPEG_DIR/lib/ffmpeg-2.18
    fi
  fi
}

get_pid() {
  OUT=/tmp/wd_sh.$$
  ps ax -o pid,args > $OUT
  PID=`cat $OUT | awk '/ffmpeg_wd.sh.*start/ {print $1}'`
  FFMPEG_PID=`cat $OUT | awk '/bin\/ffmpeg/ {print $1}'`
  rm -f $OUT
}

check_for_watchdog() {
  get_pid
  
  # Don't start if an FFMPEG Watchdog is already running
  for pid in $PID
  do
    if [ $pid != $$ ] ; then
      echo "FFMPEG watchdog already started"
      exit 0
    fi
  done
  
  # Kill any FFMPEG processes running without a watchdog
  for pid in $FFMPEG_PID
  do
    if [ $pid != $$ ] ; then
      echo "Killing orphan FFMPEG process $pid"
      kill $pid
    fi
  done
}

start_ffmpeg()
{
  if [ ! -f $ARG ] ; then
    echo "ffmpeg_wd.sh: Missing FFMPEG arguments file"
    exit 1
  fi
  . $ARG
  
  FFMPEG_DIR=/disk/cdn
  FFMPEG_BIN=bin
  case "$INPUT" in
    *rtmp:*) get_ffmpeg_bin ;;
  esac
  FFMPEG_EXE="$FFMPEG_DIR/$FFMPEG_BIN/ffmpeg -hide_banner -nostats"

  #echo $FFMPEG_EXE $RE -i "$INPUT" $ARGS -f mpegts "$OUTPUT" > /dev/console
  $FFMPEG_EXE $RE -i "$INPUT" $ARGS -f mpegts "$OUTPUT" > $LOG 2>&1 &
  FFMPEG_PID=$!
}

sigint_handler() {
  RUN=0
  echo "`date +'%b %d %T'`: Stopping hardware watchdog" > /dev/console
}

start_watchdog() {
  check_for_watchdog
  start_ffmpeg
  echo "`date +'%b %d %T'`: Starting FFMPEG watchdog" > /dev/console
  RUN=1
  trap sigint_handler 1 2 3 6 15
  while [ $RUN = 1 ]
  do
    grep invalid $LOG
    if [ $? = 0 ] ; then
      [ $FFMPEG_PID ] && kill $FFMPEG_PID 
	  echo "Restarting FFMPEG"
	  rm $LOG
	  start_ffmpeg
    fi
    pgrep ffmpeg 2>&1 > /dev/null
    if [ $? = 1 ] ; then
      echo "FFMPEG not running => restarting"
	  start_ffmpeg
    fi
    usleep 500000  # Sleep for 1/2sec
  done
  
  # If we get here, then need to stop everything
  [ "$FFMPEG_PID" != "" ] && kill $FFMPEG_PID  
}

stop_watchdog() {
  get_pid
  [ "$PID" != "" ] && kill $PID
  rm -f $LOG
  rm -f /tmp/cdn*
}

case "$1" in
  start) ARG=$2 ; start_watchdog ;;
   stop) stop_watchdog ;;
      *) echo "Usage: ffmpeg_wd.sh start|stop <ffmpeg args file>" ;;
esac

exit 0
